@@ -1,6 +1,7 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
-use deno_core::error::AnyError;
+use deno_core::anyhow::Error;
 use deno_core::AsyncRefCell;
+use deno_core::AsyncResult;
 use deno_core::CancelHandle;
 use deno_core::CancelTryFuture;
 use deno_core::JsRuntime;
@@ -11,7 +12,6 @@ use deno_core::ResourceId;
 use deno_core::ZeroCopyBuf;
 use std::cell::RefCell;
 use std::env;
-use std::io::Error;
 use std::net::SocketAddr;
 use std::rc::Rc;
 use tokio::io::AsyncReadExt;
@@ -42,7 +42,7 @@ struct TcpListener {
 }
 
 impl TcpListener {
-  async fn accept(self: Rc<Self>) -> Result<TcpStream, Error> {
+  async fn accept(self: Rc<Self>) -> Result<TcpStream, std::io::Error> {
     let cancel = RcRef::map(&self, |r| &r.cancel);
     let stream = self.inner.accept().try_or_cancel(cancel).await?.0.into();
     Ok(stream)
@@ -56,7 +56,7 @@ impl Resource for TcpListener {
 }
 
 impl TryFrom<std::net::TcpListener> for TcpListener {
-  type Error = Error;
+  type Error = std::io::Error;
   fn try_from(
     std_listener: std::net::TcpListener,
   ) -> Result<Self, Self::Error> {
@@ -77,19 +77,30 @@ struct TcpStream {
 }
 
 impl TcpStream {
-  async fn read(self: Rc<Self>, buf: &mut [u8]) -> Result<usize, Error> {
+  async fn read(self: Rc<Self>, mut buf: ZeroCopyBuf) -> Result<usize, Error> {
     let mut rd = RcRef::map(&self, |r| &r.rd).borrow_mut().await;
     let cancel = RcRef::map(self, |r| &r.cancel);
-    rd.read(buf).try_or_cancel(cancel).await
+    rd.read(&mut buf)
+      .try_or_cancel(cancel)
+      .await
+      .map_err(Error::from)
   }
 
-  async fn write(self: Rc<Self>, buf: &[u8]) -> Result<usize, Error> {
+  async fn write(self: Rc<Self>, buf: ZeroCopyBuf) -> Result<usize, Error> {
     let mut wr = RcRef::map(self, |r| &r.wr).borrow_mut().await;
-    wr.write(buf).await
+    wr.write(&buf).await.map_err(Error::from)
   }
 }
 
 impl Resource for TcpStream {
+  fn read(self: Rc<Self>, buf: ZeroCopyBuf) -> AsyncResult<usize> {
+    Box::pin(self.read(buf))
+  }
+
+  fn write(self: Rc<Self>, buf: ZeroCopyBuf) -> AsyncResult<usize> {
+    Box::pin(self.write(buf))
+  }
+
   fn close(self: Rc<Self>) {
     self.cancel.cancel()
   }
@@ -109,19 +120,12 @@ impl From<tokio::net::TcpStream> for TcpStream {
 fn create_js_runtime() -> JsRuntime {
   let mut runtime = JsRuntime::new(Default::default());
   runtime.register_op("listen", deno_core::op_sync(op_listen));
-  runtime.register_op("close", deno_core::op_sync(op_close));
   runtime.register_op("accept", deno_core::op_async(op_accept));
-  runtime.register_op("read", deno_core::op_async(op_read));
-  runtime.register_op("write", deno_core::op_async(op_write));
   runtime.sync_ops_cache();
   runtime
 }
 
-fn op_listen(
-  state: &mut OpState,
-  _: (),
-  _: (),
-) -> Result<ResourceId, AnyError> {
+fn op_listen(state: &mut OpState, _: (), _: ()) -> Result<ResourceId, Error> {
   log::debug!("listen");
   let addr = "127.0.0.1:4544".parse::<SocketAddr>().unwrap();
   let std_listener = std::net::TcpListener::bind(&addr)?;
@@ -131,20 +135,11 @@ fn op_listen(
   Ok(rid)
 }
 
-fn op_close(
-  state: &mut OpState,
-  rid: ResourceId,
-  _: (),
-) -> Result<(), AnyError> {
-  log::debug!("close rid={}", rid);
-  state.resource_table.close(rid).map(|_| ())
-}
-
 async fn op_accept(
   state: Rc<RefCell<OpState>>,
   rid: ResourceId,
   _: (),
-) -> Result<ResourceId, AnyError> {
+) -> Result<ResourceId, Error> {
   log::debug!("accept rid={}", rid);
 
   let listener = state.borrow().resource_table.get::<TcpListener>(rid)?;
@@ -153,30 +148,6 @@ async fn op_accept(
   Ok(rid)
 }
 
-async fn op_read(
-  state: Rc<RefCell<OpState>>,
-  rid: ResourceId,
-  mut buf: ZeroCopyBuf,
-) -> Result<usize, AnyError> {
-  log::debug!("read rid={}", rid);
-
-  let stream = state.borrow().resource_table.get::<TcpStream>(rid)?;
-  let nread = stream.read(&mut buf).await?;
-  Ok(nread)
-}
-
-async fn op_write(
-  state: Rc<RefCell<OpState>>,
-  rid: ResourceId,
-  buf: ZeroCopyBuf,
-) -> Result<usize, AnyError> {
-  log::debug!("write rid={}", rid);
-
-  let stream = state.borrow().resource_table.get::<TcpStream>(rid)?;
-  let nwritten = stream.write(&buf).await?;
-  Ok(nwritten)
-}
-
 fn main() {
   log::set_logger(&Logger).unwrap();
   log::set_max_level(
